( cd usr/lib ; rm -rf libhistory.so )
( cd usr/lib ; ln -sf libhistory.so.4 libhistory.so )
( cd usr/lib ; rm -rf libhistory.so.4 )
( cd usr/lib ; ln -sf libhistory.so.4.1 libhistory.so.4 )
( cd usr/lib ; rm -rf libreadline.so )
( cd usr/lib ; ln -sf libreadline.so.4 libreadline.so )
( cd usr/lib ; rm -rf libreadline.so.4 )
( cd usr/lib ; ln -sf libreadline.so.4.1 libreadline.so.4 )
# Install the info files for this package
if [ -x usr/bin/install-info ]
then
   usr/bin/install-info --info-dir=/usr/info /usr/info/rluserman.info.gz 2>/dev/null
   usr/bin/install-info --info-dir=/usr/info /usr/info/history.info.gz 2>/dev/null
   usr/bin/install-info --info-dir=/usr/info /usr/info/readline.info.gz 2>/dev/null
fi

